@@ -520,10 +520,102 @@ xwindow_resize_window(Display * display, Window window, cfgfile_item * item,
   return result;
 }
 
+/*----------------------------------------------------------------------------*/
+/* Here is a function that recursively calls itself (up to a limit) to find 
+   the window ID of the KDE desktop to draw on. This is the hierarchy we're 
+   trying to traverse:
+  
+   -> The root window
+   0 -> window with name="KDE Desktop"
+   1   -> window with no name
+   2     -> window with name="KDE Desktop" & _NET_WM_WINDOW_TYPE_DESKTOP
+   3       -> window with no name and width >= width of screen
+  
+   The last window in the hierarchy is the one to draw to.  The
+   numbers show the value of the `depth' argument.
+
+   NOTE: This code was copied from xpenguins source code base -
+   http://xpenguins.seul.org/
+*/
+static Window
+xwindow_get_kde_desktop(Display *display, int screen, Window window,
+                    Atom atom, char *atomname, int depth)
+{
+  char *name = NULL;
+  unsigned char *wintype = NULL;
+  Window winreturn = 0;
+  unsigned long nitems, bytesafter;
+  Atom actual_type;
+  int actual_format;
+  Window rootReturn, parentReturn, *children;
+  unsigned int nChildren;
+  char go_deeper = 0;
+  if (XFetchName(display, window, &name)) {
+    if (strcasecmp(name, "KDE Desktop") == 0) {
+      /* Presumably either at depth 0 or 2 */
+      if (XGetWindowProperty(display, window, atom, 0, 1,
+                             False, XA_ATOM,
+                             &actual_type, &actual_format,
+                             &nitems, &bytesafter,
+                             &wintype) == Success
+          && wintype) {
+        char *tmpatomname = XGetAtomName(display, *((Atom*)wintype));
+        if (tmpatomname) {
+          if (strcmp(atomname, tmpatomname) == 0 && depth == 2) {
+            /* OK, at depth 2 */
+            go_deeper = 1;
+          }
+          XFree((char *) tmpatomname);
+        }
+      }
+      else if (depth < 2) {
+        go_deeper = 1;
+      }
+    }
+    else if (depth == 1) {
+      go_deeper = 1;
+    }
+    XFree((char *) name);
+  }
+  else if (depth == 1) {
+    go_deeper = 1;
+  }
+  /* If go_deeper is 1 then there is a possibility that the background
+   * window is a descendant of the current window; otherwise we're
+   * barking up the wrong tree. */
+  if (go_deeper && XQueryTree(display, window, &rootReturn,
+                              &parentReturn, &children,
+                              &nChildren)) {
+    int i;
+    for (i = 0; i < nChildren; ++i) {
+      /* children[i] is now at depth 3 */
+      if (depth == 2) {
+        XWindowAttributes attributes;
+        if (XGetWindowAttributes(display, children[i], &attributes)) {
+          if (attributes.width >= DisplayWidth(display, screen)/2
+              && attributes.height > 0) {
+            /* Found it! */
+            winreturn = children[i];
+            break;
+          }
+        }
+      }
+      else if ((winreturn = xwindow_get_kde_desktop(display, screen,
+                                                children[i],
+                                                atom, atomname,
+                                                depth+1))) {
+        break;
+      }
+    }
+    XFree((char *) children);
+  }
+  return winreturn;
+}
+
 /*----------------------------------------------------------------------------*/
 /* Looks for the Nautilus desktop window to draw to, given the toplevel
- * window of the Nautilus desktop. Basically recursively calls itself
- * looking for subwindows the same size as the root window. 
+   window of the Nautilus desktop. Basically recursively calls itself
+   looking for subwindows the same size as the root window. 
 
  NOTE: This code was copied verbatim from xpenguins-2.2 source code base -
        http://xpenguins.seul.org/
@@ -560,20 +652,24 @@ xwindow_get_nautilus_desktop(Display *display, int screen, Window window,
 
 /*----------------------------------------------------------------------------*/
 /* Returns the root window, or the fake root window depending on
-   WM. Autodetect fake root for Xfce4.
+   WM. Autodetect fake root for Xfce4, nautilus and KDE.
 */
 Window
 xwindow_get_root_window(Display * display, int scr)
 {
+  int i;
   unsigned long dummy;
-  Window result = RootWindow(display,scr),
+  Window rootReturn, parentReturn, *children;
+  unsigned int nChildren;
+  Window result = 0;
+  Window root = RootWindow(display,scr),
     * toplevel = NULL;
   Atom TYPE, 
     NAUTILUS_DESKTOP_WINDOW_ID = XInternAtom(display,
 					     "NAUTILUS_DESKTOP_WINDOW_ID",
 					     False);
 
-  if (XGetWindowProperty(display, result,
+  if (XGetWindowProperty(display, root,
                          NAUTILUS_DESKTOP_WINDOW_ID,
                          0, 1, False, XA_WINDOW,
                          &TYPE, 
@@ -583,7 +679,36 @@ xwindow_get_root_window(Display * display, int scr)
                          (unsigned char **)((void*)&toplevel)) == Success
       && toplevel)
     result = xwindow_get_nautilus_desktop(display,scr,*toplevel,0);
-  return result;
+  
+  /* Next look for a virtual root or a KDE Desktop */
+  if (!result
+      && XQueryTree(display, root, &rootReturn, &parentReturn,
+		    &children, &nChildren)) {
+    Atom _NET_WM_WINDOW_TYPE = XInternAtom(display, 
+					   "_NET_WM_WINDOW_TYPE",
+					   False);
+    Atom __SWM_VROOT = XInternAtom(display, "__SWM_VROOT", False);
+      
+    for (i=0; i<nChildren && !result; ++i) {
+      Window *toplevel = (Window *) 0;
+      if (XGetWindowProperty(display, children[i],
+			     __SWM_VROOT, 0, 1, False, XA_WINDOW,
+			     &TYPE, (int*) ((void*)&dummy),
+			     &dummy, &dummy,
+			     (unsigned char **)((void*)&toplevel)) == Success
+	  && toplevel)
+	/* KDE < 2.1 */
+	result = *toplevel;   
+      else
+	/* KDE >= 2.1 */
+	result = xwindow_get_kde_desktop(display, scr, children[i],
+					 _NET_WM_WINDOW_TYPE,
+					 "_NET_WM_WINDOW_TYPE_DESKTOP",
+					 0);
+    }
+    XFree((char *) children);
+  }
+  return (result)?result:root;
 }
 
 #endif /* X_MISSING_DISPLAY , far, far above! */
